/**************************************************************
            ==================================
            Star Wars - The POV-Ray Collection
            ==================================

TIE_D.INC - 3D model description file for use with the
Persistence of Vision Ray-Tracer, v3.0

POV-Ray code copyright (c) 1998 Darcy Johnston

This model is provided for personal use only. Any use of this
model for commercial gain or profit is strictly forbidden -
please see the accompanying README.TXT and Conditions of Use
before using or distributing this model.

'Star Wars' and all associated images, designs and trademarks,
(including the designs of the vehicles and objects depicted in
the Star Wars POV-Ray Collection, except where otherwise
stated) are copyright (c) 1997 Lucasfilm Ltd.

**************************************************************/

#ifndef(engine_on) #declare engine_on = true #end
#ifndef(wing_angle) #declare wing_angle = 20 #end
#if(wing_angle < -20) #declare wing_angle = -20 #end
#if(wing_angle > 20) #declare wing_angle = 20 #end

#ifndef(tie_tex)
   #declare def_tex =
   texture {
      pigment { spotted color_map {
                        [ 0  color rgb<0.15, 0.15, 0.15> ]
                        [ 0.3  color rgb<0.45, 0.45, 0.5> ]
                        [ 1  color rgb<0.65, 0.65, 0.75> ]}
                turbulence 0.5
      }
      normal { dents 1.15 turbulence 0.5 omega 2 lambda 0.5 }
      finish { diffuse 0.5 ambient 0.2 specular 0.4 roughness 0.01 metallic }
      scale .5
   }
#end

#ifndef(glass_tex)
   #declare glass_tex = texture {
      pigment { color rgb 0 }
      finish { diffuse 0.27 ambient 0.2 reflection 0.2 specular 0.2 roughness 0.125 }
   }
#end

#declare solarpanel_tex =
texture {
   pigment {rgb 0.01}
   finish { diffuse .6 ambient 0.2 specular 1 roughness 0.08}
   normal {gradient y, 1 scale .1}
}

#declare dark_tex = texture { // Cockpit frame texture
  pigment {
    bozo
    colour_map {
      [0.0 colour rgb <0.2,0.2,0.2>]
      [0.5 colour rgb <0.25,0.25,0.25>]
      [1.0 colour rgb <0.2,0.2,0.3>]
    }
    scale 0.01
  }
  normal {dents 0.2 scale 0.01}
  finish {ambient 0.1 diffuse 0.8}
}

// Object Definitions

#declare WindowFrame =
union {
  intersection {
     torus {0.7,0.06 rotate <90,0,0> translate <0,0,1.01>}
     sphere {<0,0,0>, 1.251}
  }
  torus {0.4,0.025 rotate <90,0,0> translate <0,0,1.2>}
  cone {<0,0.7,1.036>, 0.09, <0,0.4,1.2>, 0.04}
  cylinder {<0,0.7,1.036>, <0,0.4,1.2>, 0.035 rotate z*90}
  cylinder {<0,0.7,1.036>, <0,0.4,1.2>, 0.035 rotate z*270}
  cone {<0,0.7,1.036>, 0.09, <0,0.4,1.2>, 0.04 rotate z*180}

  cylinder {<0,-0.4,1.2>, <0,0.4,1.2>, .035}
  sphere {<0,0.4,1.2>, .035}
  sphere {<0,-0.4,1.2>, .035}
  cylinder {<-0.4,0,1.2>, <0.4,0,1.2>, .035}
  sphere {<0.4,0,1.2>, .035}
  sphere {<0.4,0,1.2>, .035}
  texture {dark_tex}
}

#declare Window =
difference {
   cone{z, 0.7, z*1.21, .4 }
   box {<1, 1, 1.2>, <-1, -1, 2>}
   texture {glass_tex}
}

#declare TopHatch =
difference {
   intersection {
      union {
         sphere{y*.01, 1.25}
         cylinder{x*.1, x*-.1, 1.25 translate y*.05}
         #declare Angle = 9
         #while (Angle > -40)
            box {<.09, 0, .05>, <-0.09, .05, -.05> translate y*1.25 rotate x*Angle}
            #declare Angle = Angle - 9
         #end
      }
      cylinder{0, y*1.5, .9}
   }
   intersection {
      union {
         box {<-.4, .1, -2>, <-.2, 1.35, .25>}
         box {<.4, .1, -2>, <.2, 1.35, .25>}
         box {<-.9, .1, -2>, <-.55, 1.35, 0>}
         box {<.9, .1, -2>, <.55, 1.35, 0>}
      }
      cylinder {0, y*1.5, .8}
   }
   translate y*.015
}

#declare BottomHatch =
intersection {
   sphere {y*-.01, 1.25}
   cylinder{0, y*-1.5, .7}
   translate y*-.01
}

#declare Cannon =
union {
   cylinder {<0,0,0>, <0,0,0.4>, 0.05}
   cylinder {<0,0,0>, <0,0,0.5>, 0.04}
   difference {
      cylinder {<0,0,0>, <0,0,0.65>, 0.035}
      cylinder {<0,0,0.5>, <0,0,0.66>, 0.03}
   }
   box{<.09, .09, 0>, <-.09, -.09, .3>}
   texture {dark_tex}
}

#declare Reactor =
union {
   difference {
      cone {<0,0,-1>, .775, <0,0,-1.3>, .45}
      cone {<0,0,-1.01>, .35, <0,0,-1.35>, .65}
      box {<.3, .15, -.9>, <-.3, -.15, -1.2>}
      box {<.3, .2, -.9>, <-.1, -.1, -1.2> rotate -z*22.5 translate y*.1 scale .85}
      box {<.3, .2, -.9>, <-.1, -.1, -1.2> rotate -z*22.5 translate y*.1 scale <-1, 1, 1>*.85}
      box {<.3, -.2, -.9>, <-.1, .1, -1.2> rotate z*22.5 translate -y*.1 scale .85}
      box {<.3, -.2, -.9>, <-.1, .1, -1.2> rotate z*22.5 translate -y*.1 scale <-1, 1, 1>*.85}
   }
   cylinder {<0,0,-.99>, <0,0,-.98>, 0.76 texture {dark_tex} }
   #declare Angle = 0
   #while (Angle < 360)
      box {<.025, .025, .1>, <-.025, 0, -.1> rotate x*40 translate <0, .425, -1.125> rotate z*Angle}
      #declare Angle = Angle + 22.5
   #end
   scale .9 translate z*-.125
}

#declare engine_glow =
sphere { 0, 1
    pigment { color rgbt <1, 1, 1, 1> }
    halo {
      emitting
      spherical_mapping
      linear
      color_map {
        [ 0 color rgbt <1, 0, 0, 1> ]
        [ 1 color rgbt <1, 0, 0, -.5> ]
      }
      samples 10
    }
    hollow
  }

#declare Thruster =
union {
   box {<.09, .09, -.85>, <-.09, -.09, -1>}
   difference {
      cylinder {<0, 0, -.85>, <0, 0, -1.05>, 0.075}
      cylinder {<0, 0, -.85>, <0, 0, -1.15>, 0.055}
   }
   #if (engine_on)
      object {engine_glow scale 0.1 translate z*-1.05}
   #end
}

#declare TIE_Cockpit =
union {
   difference {
      sphere {0, 1.25}
      sphere {0, 1.2} // Hollow out sphere
      cylinder {z*-2, z*2, .7} // Cut out for Window and Engine
      box {<2, .9, 2>, <-2, 2, -2>} // Cut out for TopHatch
      box {<2, -1.1, 2>, <-2, -2, -2>} // Cut out for BottomHatch
      #declare Count = 0
      #while (Count < 180)
        torus {1.25, .015 rotate z*(Count+15)}
        #declare Count = Count + (180/6)
      #end
   }
   object {WindowFrame}
   object {Window}
   object {TopHatch}
   object {BottomHatch}
   object {Cannon translate <-.25, -.9, .6> }
   object {Cannon translate <.25, -.9, .6> }
   object {Reactor}
   object {Thruster translate -x*.75}
   object {Thruster translate x*.75}
}

#declare PylonBase1 =
difference {
   union {
      difference {
         box {<.75, .5, .325>, <0.01, -.5, -.325>}
         box {0, <1, .75, .5> rotate x*-15 translate z*.325}
         box {0, <1, .75, .5> rotate x*-15 translate z*.325 scale <1,1,-1>}
         box {0, <1, .75, .5> rotate x*-15 translate z*.325 scale <1,-1,1>}
         box {0, <1, .75, .5> rotate x*-15 translate z*.325 scale <1,-1,-1>}
      }
      cylinder {<0, 0, .325>, <1, 0, .325>, 0.01}
      cylinder {<0, 0, -.325>, <1, 0, -.325>, 0.01}
      cylinder {<0, .25, .275>, <1, .25, .275>, 0.01}
      cylinder {<0, .25, -.275>, <1, .25, -.275>, 0.01}
      cylinder {<0, -.25, .275>, <1, -.25, .275>, 0.01}
      cylinder {<0, -.25, -.275>, <1, -.25, -.275>, 0.01}
   }
   box {-z, <.5, .5, .5> rotate z*-26 translate y*.5}
   box {-z, <.3, .5, .5> rotate z*-26 translate <.5, .125, 0>}
   box {<-.3,0,-.5>, <.3, .5, .5> rotate z*-53 translate <.325, .375, 0>}
   box {-z, <.5, .5, .5> rotate z*-26 translate y*.5 scale <1,-1,1>}
   box {-z, <.3, .5, .5> rotate z*-26 translate <.5, .125, 0> scale <1,-1,1>}
   box {<-.3,0,-.5>, <.3, .5, .5> rotate z*-53 translate <.325, .375, 0> scale <1,-1,1>}
   box {<0, 1, 1>, <1, -1, -1> translate x*.7}
}

#declare PylonBase2 =
union {
  difference {
     box {<0.05, .625,.2>, <0.625, -.625,-.2>}
     box {<.04, .7, .25>, <.7, 0, -.25> rotate z*-10 translate y*.625}
     box {<.04, 0, .25>, <.7, -.7, -.25> rotate z*27 translate <.225, .625, 0>}
     box {<.01, .7, .25>, <.7, 0, -.25> rotate z*-10 translate y*.625 scale <1,-1,1>}
     box {<.01, 0, .25>, <.7, -.7, -.25> rotate z*27 translate <.225, .625, 0> scale <1,-1,1>}
  }
  difference {
     box {<0, .6, .175>, <1.1, -.6, -.175>}
     box {<-.01, .6, .2>, <1.2, 0, -.2> rotate z*-18 translate y*.6}
     box {<-.01, .6, .2>, <1.2, 0, -.2> rotate z*-18 translate y*.6 scale <1,-1,1>}
  }
}

#declare WingMount =
union {
   cone {0, 0.1, <0,0.8,0>, 0.08 scale <8,1,0.1>}
   cone {0, 0.07, <0,0.6,0>, 0.05 scale <7,.6,0.1> rotate <0,90,0>}
   cylinder {<0,.15,0>, <0,.4,0>, .3}
   difference {
      cone {0, 0.5 <0,0.15,0>, 0.3}
      #declare Count = 0
      #while (Count < 12)
         cylinder {<.5,0,0>, <.3,.15,0>, .02 rotate y*(360/12)*Count}
         #declare Count = Count +1
      #end
   }
   rotate <0,0,-90> rotate <90,0,0> scale <-1,1,1>
}

#declare TIE_Pylon =
union{
   cylinder {<1.2,0,0>, <2.95,0,0>, .25}
   cylinder {<2.35,0,0>, <2.6,0,0>, .32}
   cylinder {<1.85,0,0>, <2,0,0>, .275}
   #declare Angle = 0
   #while (Angle < 165)
      box {<.05, .025, .025>, <-.05, 0, -.025> translate <1.925,.275,0> rotate x*(Angle+100)}
      #declare Angle = Angle + 20
   #end
   object {PylonBase1 translate x*1.15}
   object {PylonBase2 translate x*1}
   object {WingMount translate x*3}
}

#declare CentrePanel =
union {
   intersection {
      cone {<0,-0.001,0>, 0.3, <0,0.0501,0>, 0.25 scale <1,1,100> rotate <0,25,0>}
      cone {<0,-0.001,0>, 0.3, <0,0.0501,0>, 0.25 scale <1,1,100> rotate <0,-25,0>}
      cone {<0,0,0>, 0.2, <0,0.05,0>, 0.15 scale <100,1,2> rotate <0,0,0>}
   }
   intersection {
      cone {<0,-0.001,0>, 0.25, <0,0.0501,0>, 0.2 scale <0.5,0.2,50> rotate <0,25,0>}
      cone {<0,-0.001,0>, 0.25, <0,0.0501,0>, 0.2 scale <0.5,0.2,50> rotate <0,-25,0>}
      cone {<0,0,0>, 0.2, <0,0.05,0>, 0.15 scale <50,0.2,1> rotate <0,0,0>}
      translate <0,0.05,0>
   }
   cylinder {<0,0,0>, <0,0.08,0>, 0.1}
   cone {<0,0.08,0>, 0.1, <0,0.1, 0>, 0.05}
}

#declare Sensor =
difference {
   cylinder {<0,0.08,0.5>, <0,0.08,0.95>, 0.05}
   cylinder {<0,0.08,0.5>, <0,0.08,1.1>, 0.03}
}

#declare A = <.025, 1.575, 4.7>
#declare B = <.025, .8, 4.7>
#declare C = <.025, 1.575, .3>
#declare D = <.025, .8, .3>

#declare E = <.025, 1.575, -.3>
#declare F = <.025, .8, -.3>
#declare G = <.025, 1.575, -1.45>
#declare H = <.025, .8, -1.45>

#declare R = 0.05

#declare TopPanel =
union {
   sphere {A, R}
   sphere {B, R}
   sphere {C, R}
   sphere {D, R}
   sphere {E, R}
   sphere {F, R}
   sphere {G, R}
   sphere {H, R}
   cylinder {A, C, R}
   cylinder {A, B, R}
   cylinder {D, C, R}
   cylinder {D, B, R}
   cylinder {E, F, R}
   cylinder {E, G, R}
   cylinder {H, F, R}
   cylinder {H, G, R}
   cylinder {<0.025, 1.725, 4.85>, <0.025, 1.725, -1.6>, 0.025}
   cylinder {<0.025, .65, 4.85>, <0.025, .65, 0>, 0.025}
   mesh{
      triangle {A, B, C}
      triangle {B, C, D}
      triangle {E, F, G}
      triangle {F, G, H}
      texture {solarpanel_tex}
   }
}

#declare TopWing =
union {
   box {<.025, 1.875, 5>, <-.025, .5, -1.75>}
   object {TopPanel}
   object {TopPanel scale <-1, 1, 1>}
   //scale <1, .75, .75> translate <0,.2,-.45>
}

#declare A = <0.025, .5, -.5>
#declare B = <0.025, .25, -.5>
#declare C = <0.025, .5, -1.5>
#declare D = <0.025, .25, -1.5>
#declare E = <0.025, -.25, -.5>
#declare F = <0.025, -.5, -.5>
#declare G = <0.025, -.25, -1.5>
#declare H = <0.025, -.5, -1.5>

#declare InnerCentrePanels =
union {
   sphere {A, R}
   sphere {B, R}
   sphere {C, R}
   sphere {D, R}
   sphere {E, R}
   sphere {F, R}
   sphere {G, R}
   sphere {H, R}
   cylinder {A, C, R}
   cylinder {A, B, R}
   cylinder {D, C, R}
   cylinder {D, B, R}
   cylinder {E, F, R}
   cylinder {E, G, R}
   cylinder {H, F, R}
   cylinder {H, G, R}
   mesh{
      triangle {A, B, C}
      triangle {B, C, D}
      triangle {E, F, G}
      triangle {F, G, H}
      texture {solarpanel_tex}
   }
}

#declare CentrePanels =
union {
   sphere {A, R}
   sphere {B, R}
   sphere {C, R}
   sphere {D, R}
   sphere {E, R}
   sphere {F, R}
   sphere {G, R}
   sphere {H, R}
   cylinder {A, C, R}
   cylinder {A, F, R}
   cylinder {H, F, R}
   cylinder {H, C, R}
   mesh{
      triangle {A, F, C}
      triangle {F, C, H}
      texture {solarpanel_tex}
   }
}

#declare CentreWing =
union {
   box {<0.025, .8, 1.5>, <-0.025, -.8, -1.75>}
   object {CentrePanels}
   object {InnerCentrePanels scale <-1, 1, 1>}
   //scale <1,.75,1>
}

#declare TIE_Wing =
union {
   object {CentrePanel rotate z*-90 scale <1.25, 1.25, 1.75>}
   object {TopWing}
   object {TopWing scale <1,-1,1>}
   object {CentreWing}
   difference {
      sphere {0, 0.5 scale <1, 1, 1>}
      box {<1,1,1>, <0,-1,-1>}
   }
   //object {Sensor}
   //object {Sensor scale <1,-1,1>}
   rotate z*wing_angle translate x*3.2
}

#declare Tie_D =
union{
   object{TIE_Cockpit}
   object{TIE_Pylon}
   object{TIE_Pylon scale <-1,-1,1>}
   object{TIE_Wing}
   object{TIE_Wing scale <-1,1,1>}
   texture {def_tex}
   scale 0.7
}
